/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package colorclicker4;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 *
 * @author pinter
 */
public class BoardGUI {

    private ArrayList<ArrayList<JButton>> buttons;
    private Board board;
    private int clickNum = 0;
    private Random random;

    private JPanel boardPanel;
    
    private JLabel timeLabel;
    private Timer timer;
    
    private final long startTime;

    private final int NUM_COLORED_FIELDS = 4;

    public BoardGUI(int boardSize) {
        buttons = new ArrayList<>();
        board = new Board(boardSize);
        random = new Random();
        boardPanel = new JPanel();
        boardPanel.setLayout(new GridLayout(board.getBoardSize(),
                board.getBoardSize()));
        for (int i = 0; i < board.getBoardSize(); ++i) {
            ArrayList<JButton> row = new ArrayList<>();
            for (int j = 0; j < board.getBoardSize(); ++j) {
                JButton button = new JButton("  ");
                button.setPreferredSize(new Dimension(80, 40));
                button.addActionListener(new ButtonListener(i, j));
                row.add(button);
                boardPanel.add(button);
            }
            buttons.add(row);
        }
        timeLabel = new JLabel(" ");
        timeLabel.setHorizontalAlignment(JLabel.RIGHT);
        timer = new Timer(10, new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                timeLabel.setText(elapsedTime() + " ms");
            }
        });
        startTime = System.currentTimeMillis();
        timer.start();
    }

    public JPanel getBoardPanel() {
        return boardPanel;
    }

    public void refresh() {
        for (int i = 0; i < board.getBoardSize(); i++) {
            for (int j = 0; j < board.getBoardSize(); j++) {
                refresh(i, j);
            }
        }
    }

    public void refresh(int x, int y) {
        JButton button = buttons.get(x).get(y);
        Field field = board.get(x, y);
        button.setBackground(field.getColor());
        if (field.getColor() != null) {
            button.setText(String.valueOf(field.getNumber()));
        } else {
            button.setText("");
        }
    }

    public long elapsedTime() {
        return System.currentTimeMillis() - startTime;
    }

    class ButtonListener implements ActionListener {

        private int x, y;

        public ButtonListener(int x, int y) {
            this.x = x;
            this.y = y;
        }

        @Override
        public void actionPerformed(ActionEvent e) {
            if (board.get(x, y).getColor() == null) {
                Color color = new Color(random.nextInt(256),
                        random.nextInt(256),
                        random.nextInt(256));
                board.get(x, y).setColor(color);
                board.get(x, y).setNumber(++clickNum);
                for (int i = 0; i < NUM_COLORED_FIELDS;) {
                    int x = random.nextInt(board.getBoardSize());
                    int y = random.nextInt(board.getBoardSize());
                    if (board.get(x, y).getColor() == null) {
                        board.get(x, y).setColor(color);
                        board.get(x, y).setNumber(clickNum);
                        refresh(x, y);
                        i++;
                    }
                }
                refresh(x, y);
                if (board.isOver()) {
                    timer.stop();
                    JOptionPane.showMessageDialog(boardPanel, "Nyertel " + elapsedTime()
                            + " ms alatt!", "Gratulalunk!", JOptionPane.PLAIN_MESSAGE);
                }
            }
        }

    }

    public JLabel getTimeLabel() {
        return timeLabel;
    }        

}
